﻿using System;
using System.Linq.Expressions;
using System.Reflection;
using System.ServiceModel;
using sjtu.wcf.demo.client.configs;

namespace sjtu.wcf.demo.client
{
    /// <summary>
    /// Wcf客户端
    /// </summary>
    /// <typeparam name="T">ServiceContract接口</typeparam>
    public class WcfClient<T> where T : class
    {

        private readonly string assemblyName;
        private readonly string implTypeName;
        private readonly string callType;

        public WcfClient()
        {
            callType = ConfigHelper.CallType.ToLower();
            if (callType == CallType.Local.ToString().ToLower())
            {
                assemblyName = ConfigHelper.AssemblyName;
                implTypeName = ConfigHelper.ServiceTypeName;
            }
        }

        /// <summary>
        /// 对外提供的Call方法
        /// </summary>
        /// <typeparam name="R"></typeparam>
        /// <param name="expression"></param>
        /// <returns></returns>
        public R Call<R>(Expression<Func<T, R>> expression)
        {
            if (callType == CallType.Local.ToString().ToLower())
            {
                return InvokeLocalMethod<R>(expression);
            }
            return InvokeRemoteMethod<R>(expression);
        }

        /// <summary>
        /// 调用本地程序集方法
        /// </summary>
        /// <typeparam name="R"></typeparam>
        /// <param name="operation"></param>
        /// <returns></returns>
        private R InvokeLocalMethod<R>(Expression<Func<T, R>> operation)
        {
            Assembly asm = Assembly.Load(new AssemblyName(assemblyName));
            T t = (T)asm.CreateInstance(implTypeName);
            R result = operation.Compile().Invoke(t);
            return result;
        }

        /// <summary>
        /// 调用远程wcf方法
        /// </summary>
        /// <typeparam name="R"></typeparam>
        /// <param name="operation"></param>
        /// <returns></returns>
        private R InvokeRemoteMethod<R>(Expression<Func<T, R>> operation)
        {
            ChannelFactory<T> channelFactory = new ChannelFactory<T>("*");

            T channel = channelFactory.CreateChannel();
            var client = (IClientChannel)channel;
            client.Open();
            R result = operation.Compile().Invoke(channel);
            try
            {
                if (client.State != CommunicationState.Faulted)
                {
                    client.Close();
                }
            }
            catch
            {
                client.Abort();
            }
            return result;
        }
    }
}
